/*
    Copyright 2007 Luigi Auriemma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
#else
    #define stricmp strcasecmp
    #define stristr strcasestr
#endif

typedef unsigned char   u8;



#define VER     "0.1"



int fgetz(u8 *data, int len, FILE *fd);
void reach_mail_pwd(u8 *pwd);
int hex2byte(u8 *hex);
void std_err(void);



int main(int argc, char *argv[]) {
    FILE    *fd;
    u8      buff[4096],
            *fname,
            *p;


    setbuf(stdout, NULL);

    fputs("\n"
        "Reach-a-Mail passwords decoder "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\n"
            "Usage: %s <Reach-a-Mail.ini>\n"
            "\n", argv[0]);
        printf("\n  press RETURN to exit\n");
        fgetc(stdin);
        exit(1);
    }

    fname = argv[1];
    printf("- open %s\n", fname);
    fd = fopen(fname, "rb");
    if(!fd) std_err();

    while(!fgetz(buff, sizeof(buff), fd)) {
    	p = strchr(buff, '=');
    	if(!p) continue;
    	*p++ = 0;

    	if(stristr(buff, "user")) {
    	    printf("\n  %s: %s\n", buff, p);
    	} else if(stristr(buff, "pass")) {
            reach_mail_pwd(p);
    	    printf("  %s: %s\n", buff, p);
    	}
    }

    fclose(fd);
    printf("\n  press RETURN to exit\n");
    fgetc(stdin);
    return(0);
}



int fgetz(u8 *data, int len, FILE *fd) {
    u8      *p;

    if(!fgets(data, len, fd)) return(-1);
    for(p = data; *p && (*p != '\n') && (*p != '\r'); p++);
    *p = 0;
    return(0);
}



void reach_mail_pwd(u8 *pwd) {
    int     i,
            j,
            len,
            c;

    len = hex2byte(pwd);
    for(i = len - 1; i >= 0; i--) {
        for(j = 0; j < len; j++) {
            if(i == j) continue;
            pwd[i] = (pwd[i] - pwd[j]) - j;
        }
    }
    for(i = 0; i < len; i++) {
        pwd[i] += i + 1;
    }
    pwd[i] = 0;
    len = hex2byte(pwd);

    for(i = len - 2; i >= 0; i--) {
        pwd[i] -= pwd[i + 1];
    }
    pwd[len] = 0;

    len = hex2byte(pwd);
    pwd[len] = 0;

    for(i = 0; i < (len / 2); i++) {
        c = pwd[i];
        pwd[i] = pwd[len - 1 - i];
        pwd[len - 1 - i] = c;
    }
}



int hex2byte(u8 *hex) {
    int     i;
    static const u8 hextable[255] =
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x00\x00\x00\x00\x00\x00"
        "\x00\x0a\x0b\x0c\x0d\x0e\x0f\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x0a\x0b\x0c\x0d\x0e\x0f\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00"
        "\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00";

    for(i = 0; hex[(i << 1)] && hex[(i << 1) + 1]; i++) {
        hex[i] = (hextable[hex[(i << 1)]] << 4) | hextable[hex[(i << 1) + 1]];
    }
    return(i);
}



void std_err(void) {
    perror("\nError");
    printf("\n  press RETURN to exit\n");
    fgetc(stdin);
    exit(1);
}


